import os

from django.test import TestCase

# Create your tests here.
from employee_info.models import Resource, Employment

from employee_info.load_data.load_resources import LoadResources


class ResourceTestCase(TestCase):
    def setUp(self) -> None:
        folder = os.path.dirname(__file__)
        load = LoadResources(os.path.join(folder, 'test_data', 'stamdata_multi.xml'), 'AK')
        load.load()

    def testMainPosition(self):
        resource = Resource.objects.get(resourceId=53453)
        main = resource.main_position()
        self.assertIsInstance(main, Employment)



from django.urls import path

from .. import views

app_name = 'employee_info_autocomplete'

urlpatterns = [
    path('cost_center', views.autocomplete.cost_center, name='autocomplete_cost_center'),
    path('function', views.autocomplete.function, name='autocomplete_function'),
    path('work_place', views.autocomplete.work_place, name='autocomplete_work_place')
]



from datetime import datetime

from django.core.management.base import BaseCommand
from employee_info.models import Company

from employee_info.load_data.LoadOrganisations import LoadOrganisations

now = datetime.now()


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('company', nargs='+', type=str)

    def handle(self, *args, **options):
        company = options['company'][0]
        if company == 'all':
            companies = Company.objects.all()
            companies = list(companies.values_list('companyCode', flat=True))
        else:
            companies = [company]

        for company_code in companies:
            file = '/home/datagrunnlag/Stamdata3_teis_%s.xml' % company_code

            load = LoadOrganisations(file, company_code)
            load.load()



from django.db import IntegrityError
from django.db.models import ProtectedError
from stamdata3.exceptions import InvalidRelation
from .load_data import LoadData
from ..models import Resource, Employment, Function, CostCenter, WorkPlace, Organisation
from stamdata3.Resource import Resource as Resource_stamdata


class LoadResources(LoadData):
    def load(self):
        orphans = Resource.objects.filter(company__companyCode=self.company_code)
        resources = self.stamdata.resources()
        for resource in resources:
            if resource.company_code != self.company_code:
                raise ValueError('Company code %s in file, %s requested' % (resource.company_code, self.company_code))
            resource_obj, created = Resource.objects.get_or_create(
                company=self.company,
                resourceId=resource.resource_id)

            resource_obj.firstName = resource.first_name
            resource_obj.lastName = resource.last_name
            resource_obj.socialSecurityNumber = resource.ssn
            resource_obj.status = resource.status

            resource_obj.save()
            orphans = orphans.exclude(id=resource_obj.id)

            self.load_employments(resource, resource_obj)
        for orphan in orphans:
            try:
                orphan.delete()
            except ProtectedError:
                print('Protected relations: %s' % orphan)
            except IntegrityError as e:
                print('Error deleting %s: %s' % (orphan, e))

    def load_employments(self, resource: Resource_stamdata, resource_obj: Resource):
        for employment in resource.employments:
            try:
                emp = Employment.objects.get(resource=resource_obj, id=employment.sequence_ref)
            except Employment.DoesNotExist:
                emp = Employment(resource=resource_obj, id=employment.sequence_ref)

            emp.employmentType = employment.type
            emp.employmentTypeDescription = employment.type_description
            emp.mainPosition = employment.main_position
            emp.percentage = employment.percentage
            emp.postId = employment.post_id
            emp.postIdDescription = employment.post_id_description
            emp.postCode = employment.post_code
            emp.postCodeDescription = employment.post_code_description

            try:
                emp.function = self.load_function(employment)
            except InvalidRelation as e:
                print(e)

            try:
                emp.costCenter = self.load_cost_center(employment)
            except InvalidRelation as e:
                print(e)

            try:
                emp.workPlace = self.load_work_place(employment)
            except InvalidRelation:
                pass

            try:
                emp.organisation = self.load_organisation(employment)
            except InvalidRelation as e:
                print(e)

            emp.dateFrom = employment.date_from
            emp.dateTo = employment.date_to
            emp.save()

    def load_function(self, employment):
        function = employment.relation('FUNCTION')
        function, created = Function.objects.get_or_create(company=self.company, value=function.value,
                                                           defaults={'description': function.description})
        return function

    def load_cost_center(self, employment):
        relation = employment.relation('COST_CENTER')
        cost_center, created = CostCenter.objects.get_or_create(
            company=self.company,
            value=relation.value,
            defaults={'description': relation.description})
        return cost_center

    def load_work_place(self, employment):
        relation = employment.relation('WORK_PLACE')
        workplace, created = WorkPlace.objects.get_or_create(
            company=self.company,
            value=relation.value,
            defaults={'description': relation.description})
        return workplace

    def load_organisation(self, employment):
        relation = employment.relation('ORGANIZATIONAL_UNIT')
        try:
            return Organisation.objects.get(
                company=self.company,
                orgId=relation.value)

        except Organisation.DoesNotExist:
            print('Organisation %s does not exist' % relation.value)



from django.contrib.auth.decorators import permission_required
from django.shortcuts import render, redirect

from employee_info.models import Resource, Company, Organisation


def index(request):
    return render(request, 'employee_info/index.html', {'companies': Company.objects.all()})


@permission_required('employee_info.view_employment')
def resource(request):
    employee_num = request.GET.get('employee')
    if not employee_num:
        employee_num = request.GET.get('value')

    company = request.GET.get('company')

    if employee_num:
        resource_obj = Resource.objects.get(company__companyCode=company,
                                            resourceId=employee_num)
        return render(request, 'employee_info/resource.html',
                      {'resource': resource_obj,
                       'title': 'Ansatt',
                       'companies': Company.objects.all()})
    else:
        return redirect('employee_info_browser:index')


@permission_required('employee_info.view_organisation')
def organisation(request):
    company = request.GET.get('company')
    organisation_num = request.GET.get('value')
    if not company:
        return render(request, 'employee_info/select_company.html', {'title': 'Velg firma',
                                                                     'companies': Company.objects.all()
                                                                     })
    elif not organisation_num:
        return render(request, 'employee_info/select_organisation.html',
                      {'title': 'Velg organisasjonsenhet',
                       'organisations': Organisation.objects.filter(company__companyCode=company),
                       'company': company})

    organisation_obj = Organisation.objects.get(company__companyCode=company, orgId=organisation_num)
    return render(request, 'employee_info/organisation.html', {'organisation': organisation_obj, 'company': company})



from django.contrib.auth.decorators import permission_required
from django.db.models import Model
from django.http import HttpResponseBadRequest, JsonResponse
from typing import Type

from employee_info.models import CostCenter, Function, WorkPlace


def __relation_autocomplete(model: Type[Model], company_code, search_value):
    output = []
    query_set = model.objects.filter(company__companyCode=company_code, value__startswith=search_value)

    for value in query_set:
        output.append({'value': value.value, 'label': '%s %s' % (value.value, value.description)})

    return output


def __autocomplete(request, model: Type[Model], limit=1):
    company_code = request.GET.get('company')
    if not company_code:
        return HttpResponseBadRequest('Company must be specified')
    search = request.GET.get('term')
    if search and len(search) >= limit:
        values = __relation_autocomplete(model, company_code, search)
        return JsonResponse(values, safe=False)
    else:
        return HttpResponseBadRequest('At least %d characters must be provided' % limit)


@permission_required('employee_info.view_cost_center')
def cost_center(request):
    return __autocomplete(request, CostCenter, 2)


@permission_required('employee_info.view_function')
def function(request):
    return __autocomplete(request, Function)


@permission_required('employee_info.view_work_place')
def work_place(request):
    return __autocomplete(request, WorkPlace)



from employee_info.models import Company
from stamdata3.stamdata3 import Stamdata3


class LoadData:
    def __init__(self, file, company_code):
        self.company_code = company_code
        self.stamdata = Stamdata3(file)
        self.company, created = Company.objects.get_or_create(
            companyCode=company_code)



from django.db import models


class Company(models.Model):
    companyCode = models.CharField('firma', max_length=2)

    def __str__(self):
        return self.companyCode

    class Meta:
        verbose_name = 'firma'
        verbose_name_plural = 'firmaer'


class Resource(models.Model):
    company = models.ForeignKey(Company, on_delete=models.PROTECT, verbose_name='firma', related_name='resource')
    resourceId = models.IntegerField('ressursnummer')
    firstName = models.CharField('fornavn', max_length=200)
    lastName = models.CharField('etternavn', max_length=200)
    socialSecurityNumber = models.CharField('fødselsnummer', max_length=11)
    status = models.CharField(max_length=1)

    def manages_list(self):
        users = []
        for user in self.manages.all():
            users.append(user)
        return users

    def subordinates(self):
        employees = []
        for organisation in self.manages.all():
            for employment in organisation.employments.all():
                employees.append(employment.resource)
        return employees

    def main_position(self, assume: bool = False):
        """
        Get the resources main position
        :type assume: bool Assume that the main position is the one with the
        highest percentage if main position is not specified
        :rtype: Employment
        """
        employments = self.employments.filter(mainPosition=True).first()
        if not employments:
            if assume:
                employments = self.employments.order_by('percentage')
                return employments.last()
        else:
            return employments

    def __str__(self):
        return '%s %s' % (self.firstName, self.lastName)

    class Meta:
        verbose_name = 'ansatt'
        verbose_name_plural = 'ansatte'
        unique_together = ['company', 'resourceId']


class CostCenter(models.Model):
    company = models.ForeignKey(Company, on_delete=models.PROTECT, verbose_name='firma', related_name='cost_centers')
    value = models.CharField('verdi', max_length=4)
    description = models.CharField('beskrivelse', max_length=200, blank=True, null=True)

    def __str__(self):
        return '%s %s (%s)' % (self.value, self.description, self.company.companyCode)

    class Meta:
        verbose_name = 'ansvar'
        verbose_name_plural = 'ansvar'
        unique_together = ['company', 'value']


class WorkPlace(models.Model):
    company = models.ForeignKey(Company, on_delete=models.PROTECT, verbose_name='firma', related_name='work_places')
    value = models.CharField('verdi', max_length=4)
    description = models.CharField('beskrivelse', max_length=200)

    def __str__(self):
        return '%s %s (%s)' % (self.value, self.description, self.company.companyCode)

    class Meta:
        verbose_name = 'arbeidssted'
        verbose_name_plural = 'arbeidssteder'
        unique_together = ['company', 'value']


class Function(models.Model):
    company = models.ForeignKey(Company, on_delete=models.PROTECT, verbose_name='firma', related_name='functions')
    value = models.CharField('verdi', max_length=4)
    description = models.CharField('beskrivelse', max_length=200)

    def __str__(self):
        return '%s %s (%s)' % (self.value, self.description, self.company.companyCode)

    class Meta:
        verbose_name = 'funksjon'
        verbose_name_plural = 'funksjoner'
        unique_together = ['company', 'value']


class Organisation(models.Model):
    company = models.ForeignKey(Company, on_delete=models.PROTECT, verbose_name='firma', related_name='organisation')
    name = models.CharField('navn', max_length=200)
    orgId = models.CharField('nummer', max_length=6)
    parent = models.ForeignKey('self', on_delete=models.PROTECT, verbose_name='overordnet', related_name='children',
                               null=True)
    manager = models.ForeignKey(Resource, on_delete=models.PROTECT, verbose_name='leder', related_name='manages',
                                null=True)
    status = models.CharField(max_length=1)

    def __str__(self):
        return '%s %s' % (self.orgId, self.name)

    class Meta:
        verbose_name = 'organisasjonsenhet'
        verbose_name_plural = 'organisasjonsenheter'
        unique_together = ['company', 'orgId']


class Employment(models.Model):
    resource = models.ForeignKey(Resource, on_delete=models.CASCADE, verbose_name='ansatt', related_name='employments')
    employmentType = models.CharField(max_length=1)
    employmentTypeDescription = models.CharField('ansettelsestype', max_length=200)
    mainPosition = models.BooleanField('primært stillingsforhold')
    percentage = models.FloatField('prosent')
    postId = models.IntegerField()
    postIdDescription = models.CharField(max_length=200)
    postCode = models.IntegerField('stillingskode')
    postCodeDescription = models.CharField(max_length=200)

    dateFrom = models.DateField('dato fra')
    dateTo = models.DateField('dato til')

    costCenter = models.ForeignKey(CostCenter, on_delete=models.PROTECT, null=True, verbose_name='ansvar', related_name='employments')
    workPlace = models.ForeignKey(WorkPlace, on_delete=models.PROTECT, null=True, verbose_name='arbeidssted', related_name='employments')
    function = models.ForeignKey(Function, on_delete=models.PROTECT, null=True, verbose_name='funksjon', related_name='employments')
    organisation = models.ForeignKey(Organisation, on_delete=models.PROTECT, null=True,
                                     verbose_name='organisasjonsenhet', related_name='employments')

    def __str__(self):
        return '%s: %s' % (self.organisation, self.resource)

    class Meta:
        verbose_name = 'stillingsforhold'
        verbose_name_plural = 'stillingsforhold'



from django.contrib import admin
from django.db.models import Model
from typing import Type

from employee_info.models import Company, CostCenter, Employment, Organisation, Resource


@admin.register(Company)
class CompanyAdmin(admin.ModelAdmin):
    list_display = ['companyCode']


@admin.register(Resource)
class ResourceAdmin(admin.ModelAdmin):
    list_display = ['company', 'resourceId', 'firstName', 'lastName']
    list_filter = ['company']
    readonly_fields = ['manages_list', 'subordinates']


@admin.register(Organisation)
class OrganisationAdmin(admin.ModelAdmin):
    list_display = ['company', 'orgId', 'name', 'manager']
    list_filter = ['company']
    # readonly_fields = ['employments']


def value_pair_dict(model: Type[Model], key, value):
    data = model.objects.order_by().values(key, value).distinct()
    values = []
    for element in data:
        values.append((element[key],
                       '%s (%s)' % (element[value], element[key])))
    return values


class EmploymentTypeAdmin(admin.SimpleListFilter):
    title = 'Ansettelsestype'
    parameter_name = 'employmentType'

    def lookups(self, request, model_admin):
        types = Employment.objects.order_by().values('employmentType', 'employmentTypeDescription').distinct()
        values = []
        for employment_type in types:
            values.append((employment_type['employmentType'],
                           '%s (%s)' % (employment_type['employmentTypeDescription'], employment_type['employmentType'])))
        return values

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(employmentType=self.value())
        else:
            return queryset


class PostCodeFilter(admin.SimpleListFilter):
    title = 'Stillingskode'
    parameter_name = 'postCode'

    def lookups(self, request, model_admin):
        return value_pair_dict(Employment, 'postCode', 'postCodeDescription')

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(postCode=self.value())
        else:
            return queryset


@admin.register(Employment)
class EmploymentAdmin(admin.ModelAdmin):
    list_display = ['resource', 'mainPosition', 'workPlace', 'costCenter',
                    'postCodeDescription', 'employmentTypeDescription']
    list_filter = [EmploymentTypeAdmin, PostCodeFilter, 'mainPosition', 'resource__company']


@admin.register(CostCenter)
class DepartmentAdmin(admin.ModelAdmin):
    list_display = ['description', 'value', 'company']
    list_filter = ['company']



from stamdata3.exceptions import InvalidField
from .load_data import LoadData
from ..models import Organisation, Resource


class LoadOrganisations(LoadData):
    def load(self):
        organisations = self.stamdata.organisations()
        for organisation in organisations:
            org, created = Organisation.objects.get_or_create(
                company=self.company,
                orgId=organisation.id
            )

            org.name = organisation.name
            org.status = organisation.status

            if organisation.parent_id is not None:
                try:
                    org.parent = Organisation.objects.get(
                        company=self.company,
                        orgId=organisation.parent_id)
                except Organisation.DoesNotExist:
                    print('Parent organisation %s for %s not found' %
                          (organisation.parent_id, organisation.id))
            else:
                print('%s has no parent organisation' % organisation.id)

            try:
                org.manager = Resource.objects.get(
                    resourceId=organisation.manager,
                    company=self.company
                )
            except Resource.DoesNotExist:
                print('Manager %d not found' % organisation.manager)
            except InvalidField:
                print('No manager for %s' % organisation.id)

            org.save()



# Generated by Django 3.1.1 on 2020-09-13 16:09

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Company',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('companyCode', models.CharField(max_length=2, verbose_name='firma')),
            ],
            options={
                'verbose_name': 'firma',
                'verbose_name_plural': 'firmaer',
            },
        ),
        migrations.CreateModel(
            name='CostCenter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=4, verbose_name='verdi')),
                ('description', models.CharField(blank=True, max_length=200, null=True, verbose_name='beskrivelse')),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='cost_centers', to='employee_info.company', verbose_name='firma')),
            ],
            options={
                'verbose_name': 'ansvar',
                'verbose_name_plural': 'ansvar',
                'unique_together': {('company', 'value')},
            },
        ),
        migrations.CreateModel(
            name='WorkPlace',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=4, verbose_name='verdi')),
                ('description', models.CharField(max_length=200, verbose_name='beskrivelse')),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='work_places', to='employee_info.company', verbose_name='firma')),
            ],
            options={
                'verbose_name': 'arbeidssted',
                'verbose_name_plural': 'arbeidssteder',
                'unique_together': {('company', 'value')},
            },
        ),
        migrations.CreateModel(
            name='Resource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('resourceId', models.IntegerField(verbose_name='ressursnummer')),
                ('firstName', models.CharField(max_length=200, verbose_name='fornavn')),
                ('lastName', models.CharField(max_length=200, verbose_name='etternavn')),
                ('socialSecurityNumber', models.CharField(max_length=11, verbose_name='fødselsnummer')),
                ('status', models.CharField(max_length=1)),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='resource', to='employee_info.company', verbose_name='firma')),
            ],
            options={
                'verbose_name': 'ansatt',
                'verbose_name_plural': 'ansatte',
                'unique_together': {('company', 'resourceId')},
            },
        ),
        migrations.CreateModel(
            name='Organisation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='navn')),
                ('orgId', models.CharField(max_length=6, verbose_name='nummer')),
                ('status', models.CharField(max_length=1)),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='organisation', to='employee_info.company', verbose_name='firma')),
                ('manager', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='manages', to='employee_info.resource', verbose_name='leder')),
                ('parent', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='children', to='employee_info.organisation', verbose_name='overordnet')),
            ],
            options={
                'verbose_name': 'organisasjonsenhet',
                'verbose_name_plural': 'organisasjonsenheter',
                'unique_together': {('company', 'orgId')},
            },
        ),
        migrations.CreateModel(
            name='Function',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=4, verbose_name='verdi')),
                ('description', models.CharField(max_length=200, verbose_name='beskrivelse')),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='functions', to='employee_info.company', verbose_name='firma')),
            ],
            options={
                'verbose_name': 'funksjon',
                'verbose_name_plural': 'funksjoner',
                'unique_together': {('company', 'value')},
            },
        ),
        migrations.CreateModel(
            name='Employment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('employmentType', models.CharField(max_length=1)),
                ('employmentTypeDescription', models.CharField(max_length=200, verbose_name='ansettelsestype')),
                ('mainPosition', models.BooleanField(verbose_name='primært stillingsforhold')),
                ('percentage', models.FloatField(verbose_name='prosent')),
                ('postId', models.IntegerField()),
                ('postIdDescription', models.CharField(max_length=200)),
                ('postCode', models.IntegerField(verbose_name='stillingskode')),
                ('postCodeDescription', models.CharField(max_length=200)),
                ('dateFrom', models.DateField(verbose_name='dato fra')),
                ('dateTo', models.DateField(verbose_name='dato til')),
                ('costCenter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='employments', to='employee_info.costcenter', verbose_name='ansvar')),
                ('function', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='employments', to='employee_info.function', verbose_name='funksjon')),
                ('organisation', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='employments', to='employee_info.organisation', verbose_name='organisasjonsenhet')),
                ('resource', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='employments', to='employee_info.resource', verbose_name='ansatt')),
                ('workPlace', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='employments', to='employee_info.workplace', verbose_name='arbeidssted')),
            ],
            options={
                'verbose_name': 'stillingsforhold',
                'verbose_name_plural': 'stillingsforhold',
            },
        ),
    ]



from django.urls import path

from ..views import browser as views

app_name = 'employee_info_browser'

urlpatterns = [
    path('', views.index, name='index'),
    path('resource', views.resource, name='resource'),
    path('organisation', views.organisation, name='organisation'),
]



from django.urls import include, path

app_name = 'employee_info'

urlpatterns = [
    path('autocomplete/', include('employee_info.urls.autocomplete')),
    path('browser/', include('employee_info.urls.browser'))
]
